#include "../../global.h"

CentipedeVideo::CentipedeVideo(Machine *hw, ScreenOutput *disp, uint8 *cr) : VideoDevice(hw, disp, cr) {

    bg=display->createBitmap(30*8, 32*8);

    for(int i=0; i<0x3c0; i++) {

        dirty1[i]=true;
    }
}

void CentipedeVideo::update(void) {

    void *theScreen = display->getScreen();

    // First, we render the background characters.

    for(int addr=0; addr<=0x3bf; addr++) {

        // If we need to draw this character

        if(dirty1[addr]) {

			int x, y;

    	    x=(addr/32);
        	y=(31-addr%32);

            // Read the character from video RAM
       	 	int c=(machine->readMem(this, addr+0x0400));
        	c+=0x40;

            // Draw it
        	display->copyBitmap(bank[0][c], bg, x*8, y*8, ScreenOutput::solid);

            dirty1[addr]=false;
       	}
    }

    // Copy the background to the screen

    display->copyBitmap(bg, theScreen, 0, 0, ScreenOutput::solid);

    uint8 coltable[4];

    // Sprites are a bit more complex

    for(int addr=0; addr<0x10; addr++) {

        // There are 128 sprites, the odd numbers refer to the last
        // 64, the even to the first.

    	int c=0x3f&(machine->readMem(this, addr+0x7c0));
    	if(c&1) c+=127;
		c=c/2;

        int x=240-(machine->readMem(this, addr+0x7d0));
        int y=248-(machine->readMem(this, addr+0x7e0));
		int colour=machine->readMem(this, addr+0x7f0);

        if(y>=8) {

            // The sprite colours can be remapped.
            coltable[0]=0;
            coltable[1]=8-colour & 0x03;
            coltable[2]=8-(colour>>2) & 0x03;
            coltable[3]=8-(colour>>4) & 0x03;

            // Draw the sprite.  The top bit of the sprite number indicates
            // a mirrored version of the sprite.
        	display->drawSprite(bank[1][c], theScreen, x, y, ScreenOutput::transparent, coltable, 0x80 & machine->readMem(this, addr+0x7c0), 0);
        }
    }
    display->update();
}

void CentipedeVideo::writeVideoRAM(uint16 addr, uint8 data) {

        dirty1[addr]=true;
}

void CentipedeVideo::setupDisplay(void) {

    display->setMode(30*8, 32*8, 60);

    display->setPalette(0, 0x000000);
    display->setPalette(1, 0x888888);
    display->setPalette(2, 0xcccccc);
    display->setPalette(3, 0xffffff);

    display->setPalette(4, 0x000000);
    display->setPalette(5, 0x888888);
    display->setPalette(6, 0xcccccc);
    display->setPalette(7, 0xffffff);

    display->setPalette(8, 0x000000);
}

CentipedeVideo::~CentipedeVideo() {

    delete bg;
}

uint8 CentipedeVideo::lookupColour(const uint16 sprite, const uint8 colour, const uint16 bank, const uint16 palette) {

    // We remap the sprite colours on the fly, and the char colours are
    // correct anyway.
    return colour;
}

void CentipedeVideo::createBanks(void) {

	uint8 *decoded;
    int sprite, linenumber;
    uint8 *chardata[256];

    for(int i=0; i<256; i++) {

        chardata[i]=new uint8[8*8];
    }

    for(int addr=0; addr<0x0800; addr++) {

    	sprite=addr/8;
        linenumber=addr%8;

        decoded=decodePlaneData(2, addr, 0x0800);
        setSpritePixels(chardata[sprite], decoded, 0, 0, 0, linenumber, 0, 8, down);
    }

    display->createSpriteBank(chardata, bank[0], 256, 8, 8);

    for(int i=0; i<256; i++) {

        delete chardata[i];
    }

    // Sprites are 16*8

    for(int i=0; i<128; i++) {

        chardata[i]=new uint8[16*8];
    }

    for(int addr=0; addr<0x0800; addr++) {

    	sprite=addr/16;
        linenumber=addr%16;

        decoded=decodePlaneData(2, addr, 0x0800);
        setSpritePixels(chardata[sprite], decoded, 0, 1, 0, linenumber, 0, 16, down);
    }

    display->createSpriteBank(chardata, bank[1], 128, 16, 8);

    for(int i=0; i<128; i++) {

        delete chardata[i];
    }

}